/** IsomorphicDataNodeComparator.java
 *  Copyright (c) 2001 Donna Malayeri
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package edu.umd.cs.fuzzyTree;

import java.util.Comparator;
import java.util.*;

/** Compares nodes based on their isomorphism index and labels.
 *
 *  @author     Donna Malayeri
 *  @version    24-Jul-01
 */

public class IsomorphicDataNodeComparator implements Comparator
{
	DataGraph theDataGraph;

	/** Creates an IsomorphicNodeComparator using <code>dataGraph</code> for the tree. */
	IsomorphicDataNodeComparator(DataGraph dataGraph)
	{
		theDataGraph = dataGraph;
	}

	/** Two nodes are equal if they have the same label, and, if they have children,
	 *  and the same isomorphism index for each child, taking position
	 *  of children into account.
	 */

	public int compare(Object o1, Object o2)
	{
		DataNode node1 = (DataNode) o1;
		DataNode node2 = (DataNode) o2;

		if (node1.getLabel().compareTo(node2.getLabel()) != 0)
			// labels are not equal, so compare just on labels
			return node1.getLabel().compareTo(node2.getLabel());
		else  // labels are equal, so compare on isomorphism index of children
		{
			// for all children, compare isomorphism index
			Vector children1 = theDataGraph.getEdges(node1);
			Vector children2 = theDataGraph.getEdges(node2);

			// less if number of children are less, more otherwise
			if (children1.size() != children2.size())
				return children1.size() - children2.size();

			// otherwise, labels are equal, have same number of children
			// check isomorphism indices of children now

			// if they disagree on isomorphism index, return the difference of the indices
			for (int i = 0; i < children1.size(); i++)
			{
				DataNode child1 = (DataNode)children1.elementAt(i);
				DataNode child2 = (DataNode)children2.elementAt(i);
				if (child1.isomorphismIndex != child2.isomorphismIndex)
					return child1.isomorphismIndex - child2.isomorphismIndex;
			}

			// else, trees rooted at node1 and node2 are isomorphic
			return 0;
		}

	}
}